@@ -28,6 +28,9 @@ import            functools
 import            wx
 
 import fsl.utils.idle                 as idle
+import fsl.data.mesh                  as fslmesh
+import fsl.data.image                 as fslimage
+import fsleyes.data.tractogram        as tractogram
 import fsleyes_props                  as props
 import fsleyes_widgets                as fwidgets
 import fsleyes_widgets.imagepanel     as imagepanel
@@ -158,9 +161,8 @@ def _initPropertyList_VolumeOpts(threedee):
              'interpolateCmaps',
              'invert',
              'invertClipping',
-             'linkLowRanges',
-             'linkHighRanges',
-             'modulateAlpha',
+             'custom_linkRanges',
+             'custom_modulateAlpha',
              'clipImage',
              'modulateImage',
              'displayRange',
@@ -169,7 +171,7 @@ def _initPropertyList_VolumeOpts(threedee):
              'custom_overrideDataRange']
 
     if threedee:
-        plist.remove('modulateAlpha')
+        plist.remove('custom_modulateAlpha')
         plist.remove('clipImage')
         plist.remove('modulateImage')
         plist.remove('modulateRange')
@@ -199,7 +201,7 @@ def _initPropertyList_MaskOpts(threedee):
             'outlineWidth']
 
 
-def _initPropertyList_VectorOpts(threedee):
+def _initPropertyList_NiftiVectorOpts(threedee):
     return ['colourImage',
             'modulateImage',
             'clipImage',
@@ -254,9 +256,8 @@ def _initPropertyList_MeshOpts(threedee):
              'invert',
              'invertClipping',
              'discardClipped',
-             'linkLowRanges',
-             'linkHighRanges',
-             'modulateAlpha',
+             'custom_linkRanges',
+             'custom_modulateAlpha',
              'modulateData',
              'displayRange',
              'clippingRange',
@@ -318,8 +319,7 @@ def _initPropertyList_MIPOpts(threedee):
             'interpolateCmaps',
             'invert',
             'invertClipping',
-            'linkLowRanges',
-            'linkHighRanges',
+            'custom_linkRanges',
             'displayRange',
             'clippingRange',
             'window',
@@ -340,6 +340,33 @@ def _initPropertyList_VolumeRGBOpts(threedee):
             'suppressMode']
 
 
+def _initPropertyList_TractogramOpts(threedee):
+    return ['lineWidth',
+            'resolution',
+            'subsample',
+            'custom_colourMode',
+            'clipMode',
+            'custom_cmap',
+            'cmapResolution',
+            'gamma',
+            'logScale',
+            'interpolateCmaps',
+            'invert',
+            'invertClipping',
+            'custom_linkRanges',
+            'custom_modulateAlpha',
+            'displayRange',
+            'clippingRange',
+            'modulateRange',
+            'xColour',
+            'yColour',
+            'zColour',
+            'suppressX',
+            'suppressY',
+            'suppressZ',
+            'suppressMode']
+
+
 def _initWidgetSpec_Display(displayCtx, threedee):
     return {
         'name'        : props.Widget('name'),
@@ -370,14 +397,17 @@ def _initWidgetSpec_ColourMapOpts(displayCtx, threedee):
             slider=True,
             spin=True,
             showLimits=False),
-        'interpolateCmaps' : props.Widget('interpolateCmaps'),
-        'invert'           : props.Widget('invert'),
-        'invertClipping'   : props.Widget('invertClipping'),
-        'linkLowRanges'    : props.Widget('linkLowRanges'),
-        'linkHighRanges'   : props.Widget('linkHighRanges'),
-        'modulateAlpha'    : props.Widget('modulateAlpha'),
-        'logScale'         : props.Widget('logScale'),
-        'gamma'            : props.Widget(
+        'interpolateCmaps'     : props.Widget('interpolateCmaps'),
+        'invert'               : props.Widget('invert'),
+        'invertClipping'       : props.Widget('invertClipping'),
+        'linkLowRanges'        : props.Widget('linkLowRanges'),
+        'linkHighRanges'       : props.Widget('linkHighRanges'),
+        'custom_linkRanges'    : _ColourMapOpts_linkRangesWidget,
+        'logScale'             : props.Widget('logScale'),
+        'custom_modulateAlpha' : _ColourMapOpts_modulateAlphaWidget,
+        'modulateAlpha'        : props.Widget('modulateAlpha'),
+        'invertModulateAlpha'  : props.Widget('invertModulateAlpha'),
+        'gamma'                : props.Widget(
             'gamma',
             showLimits=False,
             slider=True,
@@ -529,7 +559,7 @@ def _initWidgetSpec_LabelOpts(displayCtx, threedee):
     }
 
 
-def _initWidgetSpec_VectorOpts(displayCtx, threedee):
+def _initWidgetSpec_NiftiVectorOpts(displayCtx, threedee):
     def imageName(img):
         if img is None: return 'None'
         else:           return displayCtx.getDisplay(img).name
@@ -899,6 +929,81 @@ def _initWidgetSpec_VolumeRGBOpts(displayCtx, threedee):
     }
 
 
+def _initWidgetSpec_TractogramOpts(displayCtx, threedee):
+    def cmodeName(data):
+        if data == 'orientation':
+            return 'Orientation'
+        if data is None:
+            return 'None'
+        elif isinstance(data, fslimage.Image):
+            return displayCtx.getDisplay(data).name
+        else:
+            return op.basename(data)
+
+    cmapOpts   = dict(dependencies=['colourMode'],
+                      enabledWhen=lambda o, cm: cm != 'orientation')
+    orientOpts = dict(dependencies=['colourMode'],
+                      enabledWhen=lambda o, cm: cm == 'orientation')
+    sliderOpts = dict(spin=True, slider=True, showLimits=False)
+
+    return {
+        'colourMode'        : props.Widget('colourMode', labels=cmodeName),
+        'clipMode'          : props.Widget('clipMode',   labels=cmodeName),
+        'custom_colourMode' : _TractogramOpts_colourModeWidget,
+        'xColour'           : props.Widget('xColour',      **orientOpts),
+        'yColour'           : props.Widget('yColour',      **orientOpts),
+        'zColour'           : props.Widget('zColour',      **orientOpts),
+        'suppressX'         : props.Widget('suppressX',    **orientOpts),
+        'suppressY'         : props.Widget('suppressY',    **orientOpts),
+        'suppressZ'         : props.Widget('suppressZ',    **orientOpts),
+        'suppressMode'      : props.Widget('suppressMode', **orientOpts),
+        'lineWidth'         : props.Widget('lineWidth',    **sliderOpts),
+        'resolution'        : props.Widget('resolution',   **sliderOpts),
+        'subsample'         : props.Widget('subsample',    **sliderOpts),
+
+        # We override the ColourMapOpts definitions
+        # for custom enabledWhen behaviour.
+        'useNegativeCmap'  : props.Widget('useNegativeCmap',  **cmapOpts),
+        'interpolateCmaps' : props.Widget('interpolateCmaps', **cmapOpts),
+        'invert'           : props.Widget('invert',           **cmapOpts),
+        'logScale'         : props.Widget('logScale',         **cmapOpts),
+        'invertClipping'   : props.Widget('invertClipping',   **cmapOpts),
+        'linkLowRanges'    : props.Widget('linkLowRanges',    **cmapOpts),
+        'linkHighRanges'   : props.Widget('linkHighRanges',   **cmapOpts),
+
+        'cmap' : props.Widget(
+            'cmap',
+            labels=fslcm.getColourMapLabel,
+            **cmapOpts),
+        'negativeCmap' : props.Widget(
+            'negativeCmap',
+            labels=fslcm.getColourMapLabel,
+            **cmapOpts),
+        'gamma' : props.Widget(
+            'gamma',
+            **sliderOpts,
+            **cmapOpts),
+        'cmapResolution' : props.Widget(
+            'cmapResolution',
+            **sliderOpts,
+            **cmapOpts),
+        'displayRange' : props.Widget(
+            'displayRange',
+            showLimits=False,
+            slider=True,
+            labels=[strings.choices['ColourMapOpts.displayRange.min'],
+                    strings.choices['ColourMapOpts.displayRange.max']],
+            **cmapOpts),
+        'clippingRange' : props.Widget(
+            'clippingRange',
+            showLimits=False,
+            slider=True,
+            labels=[strings.choices['ColourMapOpts.displayRange.min'],
+                    strings.choices['ColourMapOpts.displayRange.max']],
+            **cmapOpts),
+    }
+
+
 def _ColourMapOpts_ColourMapWidget(
         target,
         parent,
@@ -923,9 +1028,9 @@ def _ColourMapOpts_ColourMapWidget(
 
     loadAction.bindToWidget(panel, wx.EVT_BUTTON, loadButton)
 
-    cmap       = getWidgetSpecs(target, threedee)['cmap']
-    negCmap    = getWidgetSpecs(target, threedee)['negativeCmap']
-    useNegCmap = getWidgetSpecs(target, threedee)['useNegativeCmap']
+    cmap       = getWidgetSpecs(target, displayCtx, threedee)['cmap']
+    negCmap    = getWidgetSpecs(target, displayCtx, threedee)['negativeCmap']
+    useNegCmap = getWidgetSpecs(target, displayCtx, threedee)['useNegativeCmap']
 
     cbpanel    = imagepanel.ImagePanel(parent)
     cbpanel.SetMinSize((-1, 30))
@@ -980,6 +1085,60 @@ def _ColourMapOpts_ColourMapWidget(
     return sizer, [cmap, negCmap, useNegCmap]
 
 
+def _ColourMapOpts_modulateAlphaWidget(
+        target,
+        parent,
+        panel,
+        overlayList,
+        displayCtx,
+        threedee):
+    """Builds a panel which contains widgets for the
+    :attr:`.ColourMapOpts.modulateAlpha` and
+    :attr:`.ColourMapOpts.invertModulateAlpha` properties.
+    """
+    mod = getWidgetSpecs(target, displayCtx, threedee)['modulateAlpha']
+    inv = getWidgetSpecs(target, displayCtx, threedee)['invertModulateAlpha']
+
+    mod = props.buildGUI(parent, target, mod)
+    inv = props.buildGUI(parent, target, inv)
+
+    inv.SetLabel('Invert')
+
+    sizer = wx.BoxSizer(wx.HORIZONTAL)
+
+    sizer.Add(mod, flag=wx.EXPAND, proportion=1)
+    sizer.Add(inv, flag=wx.EXPAND, proportion=1)
+
+    return sizer, [mod, inv]
+
+
+def _ColourMapOpts_linkRangesWidget(
+        target,
+        parent,
+        panel,
+        overlayList,
+        displayCtx,
+        threedee):
+    """Builds a panel which contains widgets for the
+    :attr:`.ColourMapOpts.linkLowRanges` and
+    :attr:`.ColourMapOpts.linkHighRanges` properties.
+    """
+    lo = getWidgetSpecs(target, displayCtx, threedee)['linkLowRanges']
+    hi = getWidgetSpecs(target, displayCtx, threedee)['linkHighRanges']
+
+    lo = props.buildGUI(parent, target, lo)
+    hi = props.buildGUI(parent, target, hi)
+
+    lo.SetLabel('Low')
+    hi.SetLabel('High')
+
+    sizer = wx.BoxSizer(wx.HORIZONTAL)
+    sizer.Add(lo, flag=wx.EXPAND, proportion=1)
+    sizer.Add(hi, flag=wx.EXPAND, proportion=1)
+
+    return sizer, [lo, hi]
+
+
 def _NiftiOpts_VolumeWidget(
         target,
         parent,
@@ -991,8 +1150,8 @@ def _NiftiOpts_VolumeWidget(
     :attr:`.NiftiOpts.volume` and :attr:`.NiftiOpts.volumeDim` properties.
     """
 
-    volume    = getWidgetSpecs(target, threedee)['volume']
-    volumeDim = getWidgetSpecs(target, threedee)['volumeDim']
+    volume    = getWidgetSpecs(target, displayCtx, threedee)['volume']
+    volumeDim = getWidgetSpecs(target, displayCtx, threedee)['volumeDim']
 
     volume    = props.buildGUI(parent, target, volume)
     volumeDim = props.buildGUI(parent, target, volumeDim)
@@ -1025,8 +1184,8 @@ def _VolumeOpts_OverrideDataRangeWidget(
     """
 
     # Override data range widget
-    enable   = getWidgetSpecs(target, threedee)['enableOverrideDataRange']
-    ovrRange = getWidgetSpecs(target, threedee)['overrideDataRange']
+    enable   = getWidgetSpecs(target, displayCtx, threedee)['enableOverrideDataRange']
+    ovrRange = getWidgetSpecs(target, displayCtx, threedee)['overrideDataRange']
 
     enable   = props.buildGUI(parent, target, enable)
     ovrRange = props.buildGUI(parent, target, ovrRange)
@@ -1116,7 +1275,10 @@ def _MeshOpts_vertexDataWidget(
     data.
     """
 
-    loadAction = loadvdata.LoadVertexDataAction(overlayList, displayCtx)
+    loadAction = loadvdata.LoadVertexDataAction(overlayList,
+                                                displayCtx,
+                                                'vertexData',
+                                                fslmesh.Mesh)
     loadButton = wx.Button(parent)
     loadButton.SetLabel(strings.labels[panel, 'loadVertexData'])
 
@@ -1124,7 +1286,7 @@ def _MeshOpts_vertexDataWidget(
 
     sizer = wx.BoxSizer(wx.HORIZONTAL)
 
-    vdata = getWidgetSpecs(target, threedee)['vertexData']
+    vdata = getWidgetSpecs(target, displayCtx, threedee)['vertexData']
     vdata = props.buildGUI(parent, target, vdata)
 
     sizer.Add(vdata,      flag=wx.EXPAND, proportion=1)
@@ -1148,7 +1310,8 @@ def _MeshOpts_vertexSetWidget(
 
     loadAction = loadvdata.LoadVertexDataAction(overlayList,
                                                 displayCtx,
-                                                vertices=True)
+                                                'vertices',
+                                                fslmesh.Mesh)
     loadButton = wx.Button(parent)
     loadButton.SetLabel(strings.labels[panel, 'loadVertices'])
 
@@ -1156,7 +1319,7 @@ def _MeshOpts_vertexSetWidget(
 
     sizer = wx.BoxSizer(wx.HORIZONTAL)
 
-    vdata = getWidgetSpecs(target, threedee)['vertexSet']
+    vdata = getWidgetSpecs(target, displayCtx, threedee)['vertexSet']
     vdata = props.buildGUI(parent, target, vdata)
 
     sizer.Add(vdata,      flag=wx.EXPAND, proportion=1)
@@ -1177,8 +1340,8 @@ def _MeshOpts_LutWidget(
     """
 
     # enable lut widget
-    lut    = getWidgetSpecs(target, threedee)['lut']
-    enable = getWidgetSpecs(target, threedee)['useLut']
+    lut    = getWidgetSpecs(target, displayCtx, threedee)['lut']
+    enable = getWidgetSpecs(target, displayCtx, threedee)['useLut']
 
     lut    = props.buildGUI(parent, target, lut)
     enable = props.buildGUI(parent, target, enable)
@@ -1189,3 +1352,35 @@ def _MeshOpts_LutWidget(
     sizer.Add(lut,    flag=wx.EXPAND, proportion=1)
 
     return sizer, [enable, lut]
+
+
+def _TractogramOpts_colourModeWidget(
+        target,
+        parent,
+        panel,
+        overlayList,
+        displayCtx,
+        threedee):
+    """Builds a panel which contains a widget for controlling the
+    :attr:`.TractogramOpts.colourMode` property, and also has a button
+    which opens a file dialog, allowing the user to load vertex data files.
+    """
+
+    loadAction = loadvdata.LoadVertexDataAction(overlayList,
+                                                displayCtx,
+                                                'vertexData',
+                                                tractogram.Tractogram)
+    loadButton = wx.Button(parent)
+    loadButton.SetLabel(strings.labels[panel, 'loadVertexData'])
+
+    loadAction.bindToWidget(panel, wx.EVT_BUTTON, loadButton)
+
+    sizer = wx.BoxSizer(wx.HORIZONTAL)
+
+    cmode = getWidgetSpecs(target, displayCtx, threedee)['colourMode']
+    cmode = props.buildGUI(parent, target, cmode)
+
+    sizer.Add(cmode,      flag=wx.EXPAND, proportion=1)
+    sizer.Add(loadButton, flag=wx.EXPAND)
+
+    return sizer, [cmode]
