@@ -136,6 +136,7 @@ be displayed as:
    ~fsleyes.gl.glmip.GLMIP
    ~fsleyes.gl.gltensor.GLTensor
    ~fsleyes.gl.glsh.GLSH
+   ~fsleyes.gl.gltractogram.GLTractogram
 
 
 These objects are created and destroyed automatically by the canvas classes
@@ -194,6 +195,7 @@ import fsl.utils.idle                     as idle
 import fsl.version                        as fslversion
 from   fsl.utils.platform import platform as fslplatform
 import fsleyes_widgets                    as fwidgets
+import fsleyes.gl.highdpi                 as highdpi
 
 
 real_find_library = ctypes_util.find_library
@@ -239,7 +241,7 @@ OpenGL version.
 
 GL_COMPATIBILITY = None
 """Set in :func:`bootstrap`. String containing the target "major.minor"
-OpenGL compatibility version ("1.4" or "2.1").
+OpenGL compatibility version ("1.4", "2.1", or "3.3").
 """
 
 
@@ -273,6 +275,7 @@ def _selectPyOpenGLPlatform():
     # we have - don't do anything
     if wxver is None:
         return
+
     # GTK3 versions of wxpython>=4.1.1 use EGL
     # for GL initialisation. All older wxpython
     # versions, and all GTK2 versions, use GLX.
@@ -316,6 +319,21 @@ def glIsSoftwareRenderer():
     return sw and (not fastsw)
 
 
+def hasExtension(ext, glver=None):
+    """Wrapper around ``OpenGL.extensions.hasExtension``. Short-circuits
+    and always returns ``True`` if the OpenGL version in use is >= 3.3.
+
+    If ``glver`` is not provided, it is set to the value of
+    ``GL_COMPATIBILITY``.
+    """
+    if glver is None:
+        glver = float(GL_COMPATIBILITY)
+    if glver >= 3.3:
+        return True
+    import OpenGL.extensions as glexts
+    return glexts.hasExtension(ext)
+
+
 def bootstrap(glVersion=None):
     """Imports modules appropriate to the specified OpenGL version.
 
@@ -374,6 +392,8 @@ def bootstrap(glVersion=None):
 
     ``glmip_funcs``        The version-specific module containing functions for
                            rendering :class:`.GLMIP` instances.
+    ``gltractogram_funcs`` The version-specific module containing functions for
+                           rendering :class:`.GLTractogram` instances.
     ====================== ====================================================
 
 
@@ -382,10 +402,11 @@ def bootstrap(glVersion=None):
                     version will be used.
     """
 
-    import OpenGL.GL         as gl
-    import OpenGL.extensions as glexts
-    from . import               gl14
-    from . import               gl21
+    import OpenGL.GL             as gl
+    import fsleyes.gl.gl14       as gl14
+    import fsleyes.gl.gl21       as gl21
+    import fsleyes.gl.gl33       as gl33
+    import fsleyes.gl.extensions as glexts
 
     thismod = sys.modules[__name__]
 
@@ -403,7 +424,10 @@ def bootstrap(glVersion=None):
     # GL version
     glVersion = major + minor / 10.0
     glpkg     = None
-    if glVersion >= 2.1:
+    if glVersion >= 3.3:
+        verstr = '3.3'
+        glpkg  = gl33
+    elif glVersion >= 2.1:
         verstr = '2.1'
         glpkg  = gl21
     elif glVersion >= 1.4:
@@ -415,14 +439,14 @@ def bootstrap(glVersion=None):
     # The gl21 implementation depends on a
     # few extensions - if they're not present,
     # fall back to the gl14 implementation
-    if glpkg == gl21:
+    if verstr == '2.1':
 
         # List any GL21 extensions here
         exts = ['GL_EXT_framebuffer_object',
                 'GL_ARB_instanced_arrays',
                 'GL_ARB_draw_instanced']
 
-        if not all(map(glexts.hasExtension, exts)):
+        if not all(hasExtension(e, 2.1) for e in exts):
             log.warning('One of these OpenGL extensions is '
                         'not available: [{}]. Falling back '
                         'to an older OpenGL implementation.'
@@ -433,24 +457,26 @@ def bootstrap(glVersion=None):
     # If using GL14, and the ARB_vertex_program
     # and ARB_fragment_program extensions are
     # not present, we're screwed.
-    if glpkg == gl14:
+    if verstr == '1.4':
 
         exts = ['GL_EXT_framebuffer_object',
                 'GL_ARB_vertex_program',
                 'GL_ARB_fragment_program',
                 'GL_ARB_texture_non_power_of_two']
 
-        if not all(map(glexts.hasExtension, exts)):
+        if not all(hasExtension(e, 1.4) for e in exts):
             raise RuntimeError('One of these OpenGL extensions is '
                                'not available: [{}]. This software '
                                'cannot run on the available graphics '
                                'hardware.'.format(', '.join(exts)))
 
-        # Tensor/SH/MIP overlays are not available in GL14
+        # Tensor/SH/MIP/tractogram overlays
+        # are not available in GL14
         import fsleyes.displaycontext as dc
         dc.ALL_OVERLAY_TYPES            .remove('tensor')
         dc.ALL_OVERLAY_TYPES            .remove('sh')
         dc.ALL_OVERLAY_TYPES            .remove('mip')
+        dc.ALL_OVERLAY_TYPES            .remove('tractogram')
         dc.OVERLAY_TYPES['DTIFitTensor'].remove('tensor')
         dc.OVERLAY_TYPES['Image']       .remove('sh')
         dc.OVERLAY_TYPES['Image']       .remove('tensor')
@@ -460,20 +486,22 @@ def bootstrap(glVersion=None):
     log.debug('Using OpenGL {} implementation with renderer {}'.format(
         verstr, renderer))
 
-    thismod.GL_VERSION         = str(glVersion)
-    thismod.GL_COMPATIBILITY   = verstr
-    thismod.GL_RENDERER        = renderer
-    thismod.glvolume_funcs     = glpkg.glvolume_funcs
-    thismod.glrgbvolume_funcs  = glpkg.glrgbvolume_funcs
-    thismod.glrgbvector_funcs  = glpkg.glrgbvector_funcs
-    thismod.gllinevector_funcs = glpkg.gllinevector_funcs
-    thismod.glmask_funcs       = glpkg.glmask_funcs
-    thismod.glmesh_funcs       = glpkg.glmesh_funcs
-    thismod.gllabel_funcs      = glpkg.gllabel_funcs
-    thismod.gltensor_funcs     = glpkg.gltensor_funcs
-    thismod.glsh_funcs         = glpkg.glsh_funcs
-    thismod.glmip_funcs        = glpkg.glmip_funcs
-    thismod._bootstrapped      = True
+    # Import GL version-specific sub-modules
+    globjects = ['glvolume', 'glrgbvolume', 'glrgbvector', 'gllinevector',
+                 'glmask', 'glmesh', 'gllabel', 'gltensor', 'glsh', 'glmip',
+                 'gltractogram']
+
+    for globj in globjects:
+        modname = '{}_funcs'.format(globj)
+        setattr(thismod, modname, getattr(glpkg, modname, None))
+
+    thismod.GL_VERSION       = str(glVersion)
+    thismod.GL_COMPATIBILITY = verstr
+    thismod.GL_RENDERER      = renderer
+    thismod._bootstrapped    = True
+
+    # Initialise extensions module.
+    glexts.initialise()
 
     # If we're using a software based renderer,
     # reduce the default performance settings
@@ -483,7 +511,7 @@ def bootstrap(glVersion=None):
                   'lowering default performance settings.')
 
         import fsleyes.displaycontext as dc
-        dc.SceneOpts.performance.setAttribute(None, 'default', 1)
+        dc.SceneOpts.highDpi = False
 
 
 def getGLContext(**kwargs):
@@ -600,30 +628,36 @@ class GLContext:
                  offscreen=False,
                  other=None,
                  target=None,
+                 requestVersion=None,
                  createApp=False,
                  ready=None,
                  raiseErrors=False):
         """Create a ``GLContext``.
 
-        :arg offscreen:   On-screen or off-screen context?
+        :arg offscreen:      On-screen or off-screen context?
 
-        :arg other:       Another ``GLContext`` instance with which GL state
-                          should be shared.
+        :arg other:          Another ``GLContext`` instance with which GL state
+                             should be shared.
 
-        :arg target:      If ``other`` is not ``None``, this must be a
-                          reference to a ``WXGLCanvasTarget``, the rendering
-                          target for the new context.
+        :arg target:         If ``other`` is not ``None``, this must be a
+                             reference to a ``WXGLCanvasTarget``, the rendering
+                             target for the new context.
 
-        :arg createApp:   If ``True``, and if possible, this ``GLContext`` will
-                          create and run a ``wx.App`` so that it can create a
-                          ``wx.glcanvas.GLContext``.
+        :arg requestVersion: A tuple containing the desired (major, minor)
+                             OpenGL API version to use. If ``None``, the best
+                             possible API version will be used.a floating point
+                             number.
 
-        :arg ready:       Function which will be called when the context has
-                          been created and is ready to use.
+        :arg createApp:      If ``True``, and if possible, this ``GLContext``
+                             will create and run a ``wx.App`` so that it can
+                             create a ``wx.glcanvas.GLContext``.
 
-        :are raiseErrors: Defaults to ``False``. If ``True``, and if the
-                          ``ready`` function raises an error, that error is not
-                          caught.
+        :arg ready:          Function which will be called when the context has
+                             been created and is ready to use.
+
+        :are raiseErrors:    Defaults to ``False``. If ``True``, and if the
+                             ``ready`` function raises an error, that error is
+                             not caught.
         """
 
         def defaultReady():
@@ -640,6 +674,9 @@ class GLContext:
         self.__buffer    = None
         self.__app       = None
 
+        if requestVersion is not None:
+            requestVersion = tuple(requestVersion)
+
         osmesa     = os.environ.get('PYOPENGL_PLATFORM', None) == 'osmesa'
         canHaveGui = fwidgets.canHaveGui()
         haveGui    = fwidgets.haveGui()
@@ -665,7 +702,9 @@ class GLContext:
         # need to create a GL canvas to create
         # another one.
         if other is not None:
-            self.__createWXGLContext(other=other.__context, target=target)
+            self.__createWXGLContext(other=other.__context,
+                                     target=target,
+                                     requestVersion=requestVersion)
             ready()
             return
 
@@ -691,7 +730,7 @@ class GLContext:
 
             app = self.__app
 
-            self.__createWXGLContext()
+            self.__createWXGLContext(requestVersion=requestVersion)
 
             # If we've created and started
             # our own loop, kill it
@@ -848,17 +887,25 @@ class GLContext:
         self.__canvas.Show(True)
 
 
-    def __createWXGLContext(self, other=None, target=None):
+    def __createWXGLContext(self,
+                            other=None,
+                            target=None,
+                            requestVersion=None):
         """Creates a ``wx.glcanvas.GLContext`` object, assigning it to
         an attribute called ``__context``. Assumes that a
         ``wx.glcanvas.GLCanvas`` has already been created.
 
-        :arg other:  Another `wx.glcanvas.GLContext`` instance with which
-                     the new context should share GL state.
+        :arg other:          Another `wx.glcanvas.GLContext`` instance with
+                             which the new context should share GL state.
 
-        :arg target: If ``other`` is not ``None``, this must be a
-                     ``wx.glcanvas.GLCanvas``, the rendering target for the
-                     new context.
+        :arg target:         If ``other`` is not ``None``, this must be a
+                             ``wx.glcanvas.GLCanvas``, the rendering target
+                             for the new context.
+
+        :arg requestVersion: A tuple containing the desired (major, minor)
+                             OpenGL API version to use. If ``None``, the best
+                             possible API version will be used.a floating point
+                             number.
 
         .. warning:: This method *must* be called via the ``wx.MainLoop``.
         """
@@ -866,42 +913,72 @@ class GLContext:
         import                wx
         import wx.glcanvas as wxgl
 
-        # Versions of wxwidgets 3.1 and newer (approximately
-        # corresponding to wxpython 4.1 and newer) allow
-        # us to select a GL compatibility profile (required,
-        # because we rely on GL 1.4/2.1).
+        # Creating a GL context is a pain for various reasons.
         #
-        # This only seems to work with wayland/EGL builds of
-        # wxPython. We assume that gtk3+wxpython>=4.1.1
-        # supports requesting a compatibility profile,
-        # but don't bother for gtk2+older wxpython
-        wxver  = fwidgets.wxVersion()
-        wxplat = fwidgets.wxPlatform()
-        if wxver is not None                               and \
-           fslversion.compareVersions(wxver, '4.1.1') >= 0 and \
-           wxplat in (fwidgets.WX_MAC_COCOA, fwidgets.WX_GTK3):
-            attrs = wxgl.GLContextAttrs()
-            attrs.CompatibilityProfile()
-            attrs.EndList()
-            kwargs = {'ctxAttrs' : attrs}
-        else:
-            kwargs = {}
+        #  1. wxPython only supports requesting core/
+        #     compatiblity profiles from 4.1.1 onwards.
+        #  2. In 4.1.1, the GLContext interface changed
+        #     so that we have to create and pass a
+        #     GLContextAttrs object
+        #  3. Step 2 works on macOS, but on linux only
+        #     seems to work with GTK3/wayland/EGL builds
+        #     of wxpython.
+        #  4. In order to find out whether we can use
+        #     OpenGL 3.3, we have to create a GLContext,
+        #     and check whether it isOK().
+        #
+        # For older/non-GTK3 builds, we can't request a
+        # particular profile, so we create a GLContext
+        # with no arguments, and just hope.
+        candidates = []
+        wxver      = fwidgets.wxVersion()
+
+        if wxver is not None and \
+           fslversion.compareVersions(wxver, '4.1.1') >= 0:
+            # Request 3.3 core profile unless caller
+            # has requested an older version.
+            if requestVersion is None or requestVersion >= (3, 3):
+                coreAttrs = wxgl.GLContextAttrs()
+                coreAttrs.OGLVersion(3, 3)
+                coreAttrs.CoreProfile()
+                coreAttrs.EndList()
+                candidates.append({'ctxAttrs' : coreAttrs})
+            # Request compat profile if we
+            # can't request a core profile.
+            compatAttrs = wxgl.GLContextAttrs()
+            compatAttrs.CompatibilityProfile()
+            compatAttrs.EndList()
+            candidates.append({'ctxAttrs' : compatAttrs})
+
+        # Fall back for older wxPython where we
+        # can't use a GLContextAttrs object. In
+        # this case, we get what we're given.
+        candidates.append({})
 
         log.debug('Creating wx.GLContext')
 
-        if other is not None:
-            self.__context = wxgl.GLContext(target, other=other, **kwargs)
+        for  candidate in candidates:
+            if other is not None:
+                ctx = wxgl.GLContext(target, other=other, **candidate)
+            else:
+                ctx = wxgl.GLContext(self.__canvas, **candidate)
 
+            if not hasattr(ctx, 'IsOK'):
+                break
+            if ctx.IsOK():
+                break
         else:
-            self.__context = wxgl.GLContext(self.__canvas, **kwargs)
+            raise RuntimeError('Cannot create GL context')
 
-            # We can't set the context target
-            # until the dummy canvas is
-            # physically shown on the screen.
-            while not self.__canvas.IsShownOnScreen():
-                wx.GetApp().Yield()
+        self.__context = ctx
 
-            self.__context.SetCurrent(self.__canvas)
+        # We can't set the context target
+        # until the dummy canvas is
+        # physically shown on the screen.
+        while not self.__canvas.IsShownOnScreen():
+            wx.GetApp().Yield()
+
+        self.__context.SetCurrent(self.__canvas)
 
 
     def __createOSMesaContext(self):
@@ -1079,28 +1156,8 @@ class WXGLCanvasTarget:
     def canToggleHighDPI():
         """Return ``True`` if high-DPI scaling can be toggled, ``False``
         otherwise.
-
-        Under GTK, high DPI support is not possible with wxPython < 4.0.7, as
-        ``wx.Window.GetContentScaleFactor`` always returns 1. Under GTK and
-        from wxPython 4.0.7 onwards, GL canvases are scaled automatically.
-
-        Under macOS and wxpython < 4.1.0, high DPI must be explicitly requested
-        for GL canvases via a Cocoa API call. This can be done via the
-        :meth:`EnableHighDPI` method.
-
-        Under macOS and with wxpython >= 4.1.0, GL canvases are scaled
-        automatically.
         """
-
-        import wx
-
-        if platform.system() != 'Darwin':
-            return False
-
-        wxver = getattr(wx, '__version__', '1.0.0')
-        wxver = [int(v) for v in wxver.split('.')[:3]]
-
-        return wxver < [4, 1, 0]
+        return highdpi.needsToggling()
 
 
     @staticmethod
@@ -1184,7 +1241,7 @@ class WXGLCanvasTarget:
         self.__freezeSwapBuffers = False
         self.__context           = context
 
-        if WXGLCanvasTarget.canToggleHighDPI():
+        if highdpi.needsToggling():
             self.__dpiscale = 1.0
         else:
             self.__dpiscale = self.GetContentScaleFactor()
@@ -1256,6 +1313,13 @@ class WXGLCanvasTarget:
         raise NotImplementedError()
 
 
+    def pixelSize(self):
+        """Returns the size (width, height) of one pixel in units proportional
+        to the the display coordinate system.
+        """
+        raise NotImplementedError()
+
+
     def _draw(self, *a):
         """This method should implement the OpenGL drawing logic - it must be
         implemented by subclasses.
@@ -1440,9 +1504,8 @@ class WXGLCanvasTarget:
         """
 
         # We don't necessarily need to
-        # enable high-DPI support - see
-        # __init__.
-        if not WXGLCanvasTarget.canToggleHighDPI():
+        # enable high-DPI support
+        if not highdpi.needsToggling():
             return
 
         if not self._setGLContext():
@@ -1458,14 +1521,7 @@ class WXGLCanvasTarget:
 
         # on macOS, we have to set
         # scaling on the GL canvas
-        try:
-            import objc
-            nsview = objc.objc_object(c_void_p=self.GetHandle())
-            nsview.setWantsBestResolutionOpenGLSurface_(enable)
-
-        # objc library not present
-        except ImportError:
-            return
+        highdpi.toggle(self, enable)
 
         if enable: self.__dpiscale = scale
         else:      self.__dpiscale = 1.0
